
import java.io.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Hashtable;

import comprti.*;

/**
* It will load the given class name.
*
* @date March 7, 1998 (Updated at July 3, 1998)
* @author H. Timucin Ozdemir
* @version 0.1
*
*/
public class 
RTIComponentLoader extends ClassLoader implements DebugFlags
{
   protected Hashtable classes ;
   protected Hashtable servlets ;
   protected String path ;

   public RTIComponentLoader(String _path) {  
      classes = new Hashtable(); 
      servlets = new Hashtable(); 
      path = new String(_path);
   }// end of Constructor

   /**
   *
   */
   public synchronized RTIComponent
   loadRTIComponentObject(String string) 
   {
      RTIComponent obj = null;
      Class objClass = (Class)classes.get(string);
      if(DEBUG1) System.out.println("        >>>>>> Load class : "+string); 
      if (objClass == null)
      {
         try
         {
            objClass = loadClass(string, true);

         } catch (ClassNotFoundException e3) { objClass = null;  }
           catch (ClassCastException e4)     { objClass = null;  }

      }// end of if 

      if (objClass != null)
      {
         try
         {
            obj = (RTIComponent) objClass.newInstance();

         } catch (InstantiationException e1) { obj = null;  }
           catch (IllegalAccessException e2) { obj = null;  }

      }// end of if 

      return obj;

   }// end of loadCORBAObject()

   /**
   *
   */
   public synchronized RTIComponent
   loadObject(String string) 
   {
      RTIComponent obj = null;
      Class objClass = (Class)classes.get(string);

      if (objClass == null)
      {
         try
         {
            objClass = loadClass(string, true);

         } catch (ClassNotFoundException e3) { objClass = null;  }
           catch (ClassCastException e4)     { objClass = null;  }

      }// end of if 

      if (objClass != null)
      {
         try
         {
            obj = (RTIComponent) objClass.newInstance();

         } catch (InstantiationException e1) { obj = null;  }
           catch (IllegalAccessException e2) { obj = null;  }

      }// end of if 

      if(DEBUG1 && obj==null) System.out.println(" Object is NULL! ");
      return obj;

   }// end of loadObject()


   /**
   * This is a simple version for external clients since they
   * will always want the class resolved before it is returned
   * to them.
   */
   public Class 
   loadClass(String className) throws ClassNotFoundException 
   { return (loadClass(className, true)); }

   /**
   * This is the required version of loadClass which is called
   * both from loadClass above and from the internal function
   * FindClassFromClass.
   */
   public synchronized Class 
   loadClass(String className, boolean resolveIt) 
            throws ClassNotFoundException, ClassFormatError
   {
        Class result;
        byte  classData[];

        if(DEBUG1) System.out.println("        >>>>>> Load class : "+className);

        /*************************************/
        /** Check our local cache of classes */
        /*************************************/
        result = (Class)classes.get(className);
       
        if(result == null)
        { 
           if(DEBUG1) System.out.println("        >>>>>> Not in the cache ");
           /*******************************************/
           /** Check with the primordial class loader */
           /*******************************************/
           try 
           {  result = super.findSystemClass(className);
           } catch(ClassNotFoundException cnfe) { result  = null; }
             catch(ClassFormatError cnfe1) { result  = null; }

        }// end of if

        if (result == null) 
        {
              if(DEBUG1) System.out.println("        >>>>>> Not in the System ");
              /***************************************/
              /** Try to load it from our repository */
              /***************************************/
              classData = loadClassData(className);

              if (classData == null) throw new ClassNotFoundException();

              /*************************************/
              /** Define it (parse the class file) */
              /*************************************/
              result = defineClass(classData, 0, classData.length);

              if (result == null) throw new ClassFormatError();

              if (resolveIt) resolveClass(result);

              classes.put(className, result);

        }// end of if

        return result;

    }// end of loadClass()


   /**
   *
   * This sample function for reading class implementations reads
   * them from the local file system
   *
   */
   protected byte[] 
   loadClassData(String className) throws ClassNotFoundException 
   {
      String string1 = className.replace('.',File.separatorChar) +".class";

      File file = new File(path,string1);

      byte[] result = null;
      if(DEBUG1) System.out.println(" path is "+file.getPath());
      if(DEBUG1) System.out.println(" class is "+file.getName());

      try 
      {
         FileInputStream fi = new FileInputStream(file);
         result = new byte[fi.available()];
         fi.read(result);

      } catch (Exception e) 
        {  throw new ClassNotFoundException();  }

     return result;

   }// end of loadClassData()


}// end of Class RTIComponentLoader

 
